# Slideflow-NonCommercial - Add-ons for the deep learning library Slideflow
# Copyright (C) 2024 James Dolezal
#
# This file is part of Slideflow-NonCommercial.
#
# Slideflow-NonCommercial is licensed under the Creative Commons Attribution-NonCommercial 4.0 International License.
#
# You are free to share, copy, and redistribute the material in any medium or format, and to adapt, remix, transform, and build upon the material, as long as you follow the terms of the license.
#
# Under the following terms:
# - Attribution: You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.
# - NonCommercial: You may not use the material for commercial purposes.
#
# Slideflow-NonCommercial is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Creative Commons Attribution-NonCommercial 4.0 International License for more details.
#
# You should have received a copy of the Creative Commons Attribution-NonCommercial 4.0 International License
# along with Slideflow-NonCommercial. If not, see <https://creativecommons.org/licenses/by-nc/4.0/>.


import sys
import pkgutil

def register_extras():
    # Register the additional pretrained feature extractors
    from . import extractors
    for submodule in pkgutil.iter_modules(extractors.__path__):
        original_module = sys.modules.get(submodule.name)
        module = submodule.module_finder.find_spec(submodule.name).loader.load_module(submodule.name)
        # If an original module existed, restore it
        if original_module:
            sys.modules[submodule.name] = original_module
        else:
            # Otherwise, remove the new module from the primary namespace
            del sys.modules[submodule.name]
        sys.modules[f'slideflow.model.extractors.{submodule.name}'] = module

    # Register BISCUIT
    from . import biscuit
    sys.modules['slideflow.biscuit'] = biscuit
    
    # Register StyleGAN
    from . import stylegan2, stylegan3
    sys.modules['slideflow.gan.stylegan2'] = stylegan2
    sys.modules['slideflow.gan.stylegan3'] = stylegan3
